function [xstar, relerr_save] = bpdq_cp_iter(op,proxf1,proxf2,sigma,tau,Th,iter,verb)

% [xstar, relerr_save] = bpdq_cp_iter(op,proxf1,proxf2,sigma,tau,Th,iter)
%
% Usage:
% Chambolle-Pock algorithm for solving the convex optimization
%       xstar = argmin_x [f1(Ax) + f2(x)]
%
% Inputs:
%  * op: operator with input (x,mode). Operator A.
%  * proxf1: operator with input (z,gamma). Proximity operator of f1. Defined for vectors
%  * proxf2: operator with input (z,gamma). Proximity operator of f2. Defined for vectors
%  * sigma: scalar. Parameter associated with proxf1.
%  * tau: scalar. Parameter associated with proxf2.
%  * Th: scalar. Stop criterion. If not specified, Th = 10^-4.
%  * iter: scalar. Number of iterations. If not specified, the max number
%    of iterations will be 500.
% 
% Outputs :
%  * xstar: returned solution
%  * relerr_save: record of relative error of iterates; 
%        with relerr_save(n) = ||x_n-x_(n-1)||/||x_n||
%
% Comments:
%  Proximity operators are defined as follows: 
%       prox_{gamma f} z = argmin_y f(y) + (1/(2*gamma))*||z - y||^2
%
% References:
% Chambolle, A. & Pock, T. "A first-order primal-dual algorithm for convex
% problems with applications to imaging". Technical Report, CMAP, ??cole
% Polytechnique. May 2010.
%
% Author:
%   Adriana Gonzalez. 13-04-2011


% Initialization

if (nargin == 5 || isempty(Th))
    Th = 10^-4;
end

infit = true;

if (nargin == 6 || isempty(iter))
    iter = 500;
%    infit = true;
end

info = op([],0);
m = info{1};
n = info{2};

u = cell(2,1);
u{2} = zeros(m,1);

x_est = cell(2,1);
x_est{2} = zeros(n,1);

T = NaN*ones(iter+1,1);

v = cell(2,1);
v{2} = zeros(n,1);


% Iterations

for i = 2 : (iter + 1)

    u{1} = u{2};
    x_est{1} = x_est{2};
    v{1} = v{2};
       
    u{2} = u{1} + sigma * op(v{1},1) - sigma * proxf1(((u{1} + sigma * op(v{1},1))*(1/sigma)),1/sigma);
    x_est{2} = proxf2(real(x_est{1} - tau * op(u{2},2)),tau);
    v{2} = 2*x_est{2} - x_est{1};
    
    % Computing Convergence
    if (norm(x_est{1}) ~= 0)
        T(i) = norm(x_est{2} - x_est{1})/norm(x_est{1});
    end
 
    if (verb>=1)
      fprintf('CP iter %g, relerr = %g\n',i,T(i));
    end
    
    % Stop Criterion = Th
    if (T(i) < Th && infit)
        iter = i-1;
        break
    end
    
end

relerr_save = T(1:iter);
xstar = x_est{2};